/*
 * @(#)AbstractUnpacker.java  1.0  November 19, 2007
 *
 * Copyright (c) 2007 Werner Randelshofer
 * Staldenmattweg 2, CH-6405 Immensee, Switzerland
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Werner Randelshofer. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Werner Randelshofer.
 */

package ch.randelshofer.velcrofly;

import com.ibm.icu.text.*;
import ch.randelshofer.gui.*;
import ch.randelshofer.util.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
/**
 * Abstract base class for classes which unpack archives.
 *
 * @author Werner Randelshofer
 * @version 1.0 November 19, 2007 Created.
 */
public abstract class AbstractUnarchiver extends RunnableWorker {
    /**
     * Archive.
     */
    protected File archive;
    
    /**
     * Output directory.
     */
    protected File outputdir;
    
    /**
     * Progress view.
     */
    protected ProgressView p;
    
    /** Creates a new instance. */
    public AbstractUnarchiver(File archive) {
        this(archive, null);
    }
    /**
     * Creates a new instance.
     *
     * @param archive The archive.
     * @param outputdir the output directory. If this value is null, the
     * output is written into the same directory which contains the archive.
     */
    public AbstractUnarchiver(File archive, File outputdir) {
        this.archive = archive;
        this.outputdir = outputdir;
        
        p = new ProgressView("Extracting files from "+getArchiveName()+" archive \""+archive.getName()+"\"", "Waiting for processor...");
    }
    
    public abstract String getArchiveName();
    
    /**
     * Called on the event dispatching thread (not on the worker thread)
     * after the <code>construct</code> method has returned.
     */
    public void finished(Object result) {
        if (result != null && (result instanceof Throwable)) {
            ((Throwable) result).printStackTrace();
            JOptionPane.showConfirmDialog(
                    null,
                    ((Throwable) result).getMessage(),
                    "VelcroFly: Decompression of "+getArchiveName()+" Archive \""+archive.getName()+"\" failed",
                    JOptionPane.DEFAULT_OPTION,JOptionPane.ERROR_MESSAGE
                    );
        }
    }
}
